@@ -53,12 +53,26 @@ class IResourceConfigurator(Interface):
     pass
 
 
+class ResourceAdded(object):
+    """Event notified after a resource is added and configured."""
+    def __init__(self, config, resource):
+        self.config = config
+        self.resource = resource
+
+
 def not_allowed_view(request):
-    raise HTTPMethodNotAllowed()
+    # FIXME using raise makes tests fail
+    return HTTPMethodNotAllowed()
 
 
 @implementer(IResourceConfigurator)
 class ResourceConfigurator(object):
+    """Internal configurator of all REST resources. It is responsible for:
+
+    # Registering to pyramid routes and views associated to resources.
+    # . . .
+
+    """
 
     methods = dict(
         index='GET',
@@ -106,7 +120,8 @@ class ResourceConfigurator(object):
         try:
             cls = config.maybe_dotted(dotted_class)
         except ImportError:
-            raise ImportError('No class %s found.' % dotted_class)
+            log.error('No class %s found.', dotted_class)
+            raise
         config.scan(dotted_module)
 
         res(cls)  # decorate class to register config
@@ -175,7 +190,8 @@ class ResourceConfigurator(object):
         if config._ainfo is None:
             config._ainfo = []
 
-        self._validate_views_args_name(resource)
+        # XXX: to be refactored
+        # self._validate_views_args_name(resource)
         self._add_routes(config, resource)
         self._add_views(config, resource)
         self._add_introspectable(config, resource)
@@ -188,6 +204,7 @@ class ResourceConfigurator(object):
             resource.pattern,
             resource.item_pattern,
             )
+        config.registry.notify(ResourceAdded(config, resource))
         self._add_deferred_children(config, resource)
 
     def _add_deferred_children(self, config, parent_resource):
@@ -198,6 +215,7 @@ class ResourceConfigurator(object):
                 self._add(config, child_resource)
 
     def _validate_views_args_name(self, resource):
+        # XXX seems ugly: need refacto.
         not_last = resource.ids[:-1]
 
         expectation = {
@@ -211,21 +229,25 @@ class ResourceConfigurator(object):
             }
 
         for view_info in resource.views.itervalues():
-            argscount = view_info.view.func_code.co_argcount
-            args_name = view_info.view.func_code.co_varnames[:argscount]
+            spec = inspect.getargspec(view_info.view)
             if inspect.isfunction(view_info.view):
-                expected = tuple(
+                expected = (
                     ['context', 'request']
                     + expectation[view_info.method]
                     )
             else:
-                expected = tuple(['self'] + expectation[view_info.method])
-            if expected != args_name:
-                raise TypeError('resource=%s, view=%s, expected %s - got %s' % (
+                expected = ['self'] + expectation[view_info.method]
+            nb_args = len(spec.args) - (
+                len(spec.defaults) if spec.defaults else 0
+                )
+            received = spec.args[:nb_args]
+            if expected != received:
+                raise TypeError(
+                    'resource=%s, view=%s, expected %s - got %s' % (
                     resource,
                     view_info.method,
                     expected,
-                    args_name,
+                    received,
                     ))
 
     def _add_routes(self, config, resource):
@@ -264,8 +286,10 @@ class ResourceConfigurator(object):
                 mapper = ClassViewMapper
                 attr = view_info.view.__name__
             settings = view_info.settings.copy()
-            settings.update(self._get_view_predicates(resource, view_info.method))
-            settings.setdefault('renderer', 'json')
+            settings.update(
+                self._get_view_predicates(resource, view_info.method)
+                )
+            settings.setdefault('renderer', 'pyramid_rest_renderer')
             config.add_view(
                 view=view_info.view,
                 mapper=mapper,
@@ -276,7 +300,9 @@ class ResourceConfigurator(object):
         if not resource.singular:
             not_allowed = [m for m in self.methods if m not in resource.views]
         if resource.singular:
-            not_allowed = [m for m in self.singular_methods if m not in resource.views]
+            not_allowed = [m for m in self.singular_methods if (
+                m not in resource.views)
+                ]
 
         for method in not_allowed:
             config.add_view(
@@ -331,9 +357,26 @@ class ResourceConfigurator(object):
             request_method=self.methods[method],
             )
 
-    def resource_path(self, resource_name, *ids):
+    def resource_path(self, request, resource_name, *args):
+        """Returns route name for ``resource_name`` with args"""
         resource = self.resources[resource_name]
-        return resource.get_path(*ids)
+        route_name = resource.get_route_name(*args)
+        ids = dict(zip(
+            resource.ids[:len(args)],
+            args,
+            ))
+        return request.route_path(route_name, **ids)
+
+
+def rest_resource_url(request, resource_name, *args):
+    utility = request.registry.getUtility(IResourceConfigurator)
+    path = utility.resource_path(request, resource_name, *args)
+    return request.host_url + path
+
+
+def rest_resource_path(request, resource_name, *args):
+    utility = request.registry.getUtility(IResourceConfigurator)
+    return utility.resource_path(request, resource_name, *args)
 
 
 class ViewInfo(object):
@@ -391,6 +434,14 @@ class BaseResource(object):
 
     @reify
     def ids(self):
+        """List of identifier names for this resource.
+
+        Example::
+
+            >>> resource = Resource('application.user')
+            >>> resource.ids
+            ['application_id', 'id']
+        """
         my_id = [] if self.singular else ['id']
         if self.parent:
             return self.lineage_ids[:-1] + my_id
@@ -398,8 +449,12 @@ class BaseResource(object):
 
     @reify
     def lineage_ids(self):
-        """Return list of identifier names a child resource needs for its route
-        pattern.
+        """List of identifier names a child resource needs for its route
+        pattern. e.g.::
+
+            >>> resource = Resource('application.user.message')
+            >>> resource.parent.lineage_ids
+            ['application_id', 'user_id']
         """
         id_name = self.short_name + '_id'
         if self.parent:
@@ -422,13 +477,12 @@ class BaseResource(object):
         config = context.config.with_package(self.info.module)
         config.registry.getUtility(IResourceConfigurator)._add(config, self)
 
-    def get_path(self, *args):
-        """Generate a path (aka a 'relative URL', a URL minus the host, scheme,
-        and port) for the rest resource.
+    def get_route_name(self, *args):
+        """Returns route name associated with that resource and the arguments
+        for url generation
 
         :param args: List of ids.
         """
-        # TODO: purge this as it does not work if config is included with prefix
         if (len(args) < self.depth
             or len(args) > self.depth + 1
             or (self.singular and len(args) != self.depth)
@@ -438,21 +492,17 @@ class BaseResource(object):
                 self.depth + 1,
                 len(args),
                 ))
-        ids = dict(zip(
-            self.ids[:len(args)],
-            args,
-            ))
         if self.singular:
-            return self.pattern.format(**ids)
+            return self.route_name
 
         is_collection_path = (len(args) == self.depth)
         is_item_path = (len(args) == self.depth + 1)
 
         if is_collection_path:
-            return self.pattern.format(**ids)
+            return self.route_name
 
         if is_item_path:
-            return self.item_pattern.format(**ids)
+            return self.item_route_name
 
 
 class Resource(BaseResource):
@@ -508,6 +558,7 @@ class resource_config(BaseResource):
 
     def __call__(self, cls):
         self.info = venusian.attach(cls, self.callback, category='pyramid')
+        self.view_class = cls
         self.__doc__ = cls.__doc__
         methods = self.singular_methods if self.singular else self.methods
         views = inspect.getmembers(
